#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import math
from scipy import optimize as opt
import matplotlib.pyplot as plt

small = math.sqrt(np.finfo(float).eps)

def lineSearch(f, g, z0, delta=small):
    f0, g0 = f(z0), g(z0)
    
    def h(alpha):
        return f(z0 - alpha * g0)
    
    if np.linalg.norm(g0) < delta:
        return z0
    else:
        # Find endpoints of bracketing interval
        a, c, hcOld, maxC = 0, small, f0, 100
        while c < maxC:
            c *= 1.2
            hc = h(c)
            if hc > hcOld:
                break
            else:
                hcOld = hc
        assert c < maxC, 'Could not find a bracketing interval'
        
        result = opt.minimize_scalar(h, bracket=(a, c))
        assert result.success, result.message
        alpha = result.x
        return z0 - alpha * g0

def Rosenbrock(z):
    return 100 * (z[1] - z[0] ** 2) ** 2 + (1 - z[0]) ** 2 

def RosenGrad(z):
    return np.array([400 * (z[0] ** 3 - z[0] * z[1]) + 2 * z[0] - 2,
                     200 * (z[1] - z[0] ** 2)])

def RosenHess(z):
    a = 1200 * z[0] ** 2 - 400 * z[1] + 2
    b = -400 * z[0]
    c = 200
    return np.array([[a, b], [b, c]])


def RosenContours():
    fig, ax = plt.subplots(subplot_kw={'aspect': 'equal'})
    n = 100
    x = np.linspace(-4, 3, n)
    y = np.linspace(-4, 3, n)
    f = np.array([[Rosenbrock(np.array([a, b])) for a in x] for b in y])
    ax.contour(x, y, f, 10, colors='grey', linewidths=1)
    return fig, ax

from sklearn import datasets

def makeData():
    digits = datasets.load_digits()
    images_and_labels = list(zip(digits.images, digits.target))
    np.random.seed(3)
    data = {'x': digits.images.reshape((len(digits.images), -1)), 'y': digits.target}
    fourOrNine = (data['y'] == 4) | (data['y'] == 9)
    data['x'], data['y'] = data['x'][fourOrNine], data['y'][fourOrNine]
    data['y'][data['y'] == 4] = 0
    data['y'][data['y'] == 9] = 1
    order = np.random.permutation(len(data['y']))
    data['x'], data['y'] = data['x'][order], data['y'][order]
    N = round(len(data['y']) * 2 / 3)
    T = {'x': data['x'][:N], 'y': data['y'][:N]}
    S = {'x': data['x'][N:], 'y': data['y'][N:]}
    return T, S